local TOOLTIP_TEXT_WITH = "With buff"
local TOOLTIP_TEXT_WITHOUT = "Without"
local NOPLAYERS = "Noone"
local DELIM = ", "
local CAST_BY = "Cast by: "
local UNKNOWN_UNIT = "Unknown"

-- set up tables for equivalence checking (e.g. Forgotten Kings vs Kings, Prayer of Fortitude vs Fortitude)
-- n.b. Paladin blessings are handled in a different way
local equivTable = {
	[GetSpellInfo(1126)] = "bwKings",			-- Mark
	[GetSpellInfo(20217)] = "bwKings",			-- Kings
	[GetSpellInfo(21562)] = "bwFort",			-- Single
	[GetSpellInfo(69377)] = "bwFort",			-- Runescroll
	[GetSpellInfo(1459)] = "bwIntellect",		-- Arcane Brillaince
	[GetSpellInfo(61316)] = "bwIntellect",		-- Dalaran Brilliance
	}
	

if not BuffWhoonDB then
	BuffWhoonDB = {
		["DBver"] = 130,
		["SK"] = 2,
		["TK"] = 1,
		["SD"] = 2,
		["TD"] = 2,
		["Sort"] = 1,
	}
end

local keyList = {"Alt", "Ctrl", "Shift"}


SLASH_BUFFWHOON1, SLASH_BUFFWHOON2 = '/bw', '/buffwhoon'
function SlashCmdList.BUFFWHOON()
	local bwCol1 = "|cFF24A0FF"
	local bwCol2 = "|cFF00FF00"
	local bwCol3 = "|cFFFFFFFF"
	DEFAULT_CHAT_FRAME:AddMessage(bwCol1 .. "BuffWhoon")
	DEFAULT_CHAT_FRAME:AddMessage(" - " .. bwCol3 .. "Mouseover a (de)buff to show who is missing it in the group or raid")
	if BuffWhoonDB["TK"] < 4 then
		DEFAULT_CHAT_FRAME:AddMessage(" - " .. bwCol2 .. keyList[BuffWhoonDB["TK"]] .. ": " .. bwCol3 .. "Toggle showing who has vs who is missing the (de)buff")
	end
	if BuffWhoonDB["SK"] < 4 then
		DEFAULT_CHAT_FRAME:AddMessage(" - " .. bwCol2 .. keyList[BuffWhoonDB["SK"]] .. ": " .. bwCol3 .. "Hold down to show who cast the buff")
	end
	InterfaceOptionsFrame_OpenToCategory("BuffWhoon")
end


local function FormatPlayerColor (fpcUnit)
	local fpcClass, fpcClassFileName = UnitClass(fpcUnit)
	local fpcColor = (CUSTOM_CLASS_COLORS or RAID_CLASS_COLORS)[fpcClassFileName]
	local fpcString = format("00%02X%02X%02X", 255 * fpcColor.r, 255 * fpcColor.g, 255 * fpcColor.b)
	return ("|c" .. fpcString .. UnitName(fpcUnit))
end

local function CheckUnit (cuCompareUnit, cuMouseoverSpell, filter, cuMouseoverCaster)
	local cuHasBuff = nil
	local cuMouseoverClass
	local cuCompareClass
	local _,_,cuGroup = GetRaidRosterInfo((UnitInRaid(cuCompareUnit) or UnitInParty(cuCompareUnit)) + 1)
	
	if UnitPlayerOrPetInRaid(cuMouseoverCaster) and not UnitPlayerOrPetInParty(cuMouseoverCaster) then
		cuMouseoverClass = UnitClass (cuMouseoverCaster)
	end

	for cuLoop = 1,40 do
		local cuCompareSpell,_,_,_,_,_,_,cuCompareCaster = UnitAura(cuCompareUnit, cuLoop, filter)
		if not cuCompareSpell then break end				-- exit if we've run out of buffs to check
		if cuCompareCaster then cuCompareClass = UnitClass (cuCompareCaster) end

		if (cuCompareSpell == cuMouseoverSpell) then
			cuHasBuff = true
			break
		end
		
		if equivTable[cuMouseoverSpell] and equivTable[cuCompareSpell]then			-- test if the buff is in the equivalence table
			if equivTable[cuMouseoverSpell] == equivTable[cuCompareSpell] then
				cuHasBuff = true
				break
			end
		end
		
	end

	if (cuHasBuff and HavingToggle) then
		table.insert (suaTable, {stUnitID = cuCompareUnit, stName = UnitName(cuCompareUnit), stGroup = cuGroup, stClass = UnitClass(cuCompareUnit), stColorText = FormatPlayerColor (cuCompareUnit)})
		suaBuffCount = suaBuffCount + 1
	elseif (not cuHasBuff and not HavingToggle) then
		table.insert (suaTable, {stUnitID = cuCompareUnit, stName = UnitName(cuCompareUnit), stGroup = cuGroup, stClass = UnitClass(cuCompareUnit), stColorText = FormatPlayerColor (cuCompareUnit)})
		suaBuffCount = suaBuffCount + 1
	end
end


local function CheckToggles()
	local toggleSK
	local toggleTK = 0
	local retCaster
	local retHaving

	if IsAltKeyDown() then
		if BuffWhoonDB["SK"] == 1 then
			toggleSK = true
		end
		if BuffWhoonDB["TK"] == 1 then
			toggleTK = 1
		end
	end

	if IsControlKeyDown() then
		if BuffWhoonDB["SK"] == 2 then
			toggleSK = true
		end
		if BuffWhoonDB["TK"] == 2 then
			toggleTK = 1
		end
	end
	
	if IsShiftKeyDown() then
		if BuffWhoonDB["SK"] == 3 then
			toggleSK = true
		end
		if BuffWhoonDB["TK"] == 3 then
			toggleTK = 1
		end
	end

	if BuffWhoonDB["SD"] == 1 or toggleSK then
		retCaster = true
	end
	
	if BuffWhoonDB["TD"] + toggleTK ~= 2 then
		retHaving = true
	end

	return retHaving, retCaster
end


local function SetUnitAura(tooltip, unit, id, filter)
	local suaGroup
	local suaUnitCount
	local suaToolTipOutput
	HavingToggle, CastToggle = CheckToggles()

-- check if we're in a raid or party and exit if not
	if GetNumRaidMembers() >0 then
		suaGroup = "raid"
		suaUnitCount = GetNumRaidMembers()
	elseif GetNumPartyMembers() > 0 then
		suaGroup = "party"
		suaUnitCount =  GetNumPartyMembers()
	else
		return
	end
	
-- don't consider buffs found on people outside of our party or raid
	if (not UnitPlayerOrPetInRaid(unit) and not UnitPlayerOrPetInParty(unit) and not UnitIsPlayer(unit)) then
		return
	end

	local suaSpell,_,_,_,_,_,_,suaCaster = UnitAura(unit, id, filter)
	suaBuffCount = 0
	suaTable = {}
	
	for i = 1, suaUnitCount do
		local suaUnitID = suaGroup .. tostring(i)
		if (UnitIsConnected(suaUnitID) and not UnitIsDeadOrGhost(suaUnitID)) then
			CheckUnit (suaUnitID, suaSpell, filter, suaCaster)
		end
	end
	
	if suaGroup == "party" then
		CheckUnit ("player", suaSpell, filter, suaCaster)
	end

	if CastToggle then
		if suaCaster then
			tooltip:AddLine(CAST_BY .. FormatPlayerColor(suaCaster))
		else
			tooltip:AddLine(CAST_BY .. UNKNOWN_UNIT)
		end
	end

	if HavingToggle then
		tooltip:AddLine(TOOLTIP_TEXT_WITH .. " (" .. suaBuffCount .. "):",0,1,0)
	else
		tooltip:AddLine(TOOLTIP_TEXT_WITHOUT .. " (" .. suaBuffCount .. "):",1,0,0)
	end


	if BuffWhoonDB["Sort"] == 3 then -- long group sort
		sort(suaTable, function(a,b) return (a.stGroup .. a.stName) < (b.stGroup .. b.stName) end)
		for i = 1, 8 do
			local suaTableOutput = {}
			for k,v in pairs (suaTable) do
				if v.stGroup == i then
					table.insert (suaTableOutput, v.stColorText)
				end
			end
			if # suaTableOutput > 0 then
				local suaToolTipOutput = table.concat (suaTableOutput, DELIM)
				tooltip:AddLine("|cffffffff[" .. i .. "]:|r " .. suaToolTipOutput,_,_,_,nil)
			end
		end

	elseif BuffWhoonDB["Sort"] == 2 then -- short group sort
		sort(suaTable, function(a,b) return (a.stGroup .. a.stName) < (b.stGroup .. b.stName) end)
		for i = 1, 8 do
			local suaTableOutput = {}	
			for k,v in pairs (suaTable) do
				if v.stGroup == i then
					table.insert (suaTableOutput, v.stColorText)
				end
			end
			if # suaTableOutput > 0 then
					if not suaToolTipOutput then
						suaToolTipOutput = "|cffffffff[" .. i .. "]|r " .. table.concat (suaTableOutput, DELIM)
					else
						suaToolTipOutput = suaToolTipOutput .. " |cffffffff[" .. i .. "]|r " .. table.concat (suaTableOutput, DELIM)
					end
			end
		end
		tooltip:AddLine(suaToolTipOutput,_,_,_,true)
				
		
	else
		local suaTableOutput = {}
		sort(suaTable, function(a,b) return (a.stClass .. a.stName) < (b.stClass .. b.stName) end)
		for k,v in pairs (suaTable) do
			table.insert (suaTableOutput, v.stColorText)
		end
		
		local suaToolTipOutput = (table.concat (suaTableOutput, DELIM) or NOPLAYERS)
		tooltip:AddLine(suaToolTipOutput,_,_,_,true)
	end


	tooltip:Show()
end
		

local function SetUnitBuff(tooltip, unit, id, filter)
	SetUnitAura(tooltip, unit, id, "HELPFUL "..(filter or ""))
end

local function SetUnitDebuff(tooltip, unit, id, filter)
	SetUnitAura(tooltip, unit, id, "HARMFUL "..(filter or ""))
end


hooksecurefunc(GameTooltip, "SetUnitAura", SetUnitAura)
hooksecurefunc(GameTooltip, "SetUnitBuff", SetUnitBuff)
hooksecurefunc(GameTooltip, "SetUnitDebuff", SetUnitDebuff)