-- Options screen for BuffWhoon
-- [K=Key], [D=Default], [S= Show Caster], [T=Toggle missing / having]

if not BuffWhoonDB then return end

local panel = CreateFrame "Frame"
panel.name = "BuffWhoon"

function panel:Load()
	local header = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
	header:SetText"BuffWhoon Configuration Settings"
	header:SetPoint ("TOPLEFT", panel, "TOPLEFT", 10, -15)
	
	local footer = panel: CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
	footer:SetText "A lightweight addon that shows who is missing raid / party buffs on mouseover"
	footer: SetPoint ("BOTTOMLEFT", panel, "BOTTOMLEFT", 10, 15)
	
	local dropdownSK = CreateFrame("Frame", "dropdownSK", panel, "UIDropDownMenuTemplate")
	local labelShow = dropdownSK:CreateFontString()
	local labelToggle = dropdownSK:CreateFontString()
	
	labelShow:SetFontObject(GameFontHighlight)
	labelShow:SetText"Show Caster Name"
	labelShow:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 0, -80)
	
	labelToggle:SetFontObject(GameFontHighlight)
	labelToggle:SetText"Missing or Having Buffs"
	labelToggle:SetPoint("LEFT", labelShow, "LEFT", 0, -50)
	
	dropdownSK:SetPoint("LEFT", labelShow, "RIGHT", 200,0)
	UIDropDownMenu_SetWidth(dropdownSK, 70)
	
	local dropdownSD = CreateFrame("Frame", "dropdownSD", panel, "UIDropDownMenuTemplate")
	dropdownSD:SetPoint("RIGHT", dropdownSK, "LEFT", 0,0)
	UIDropDownMenu_SetWidth(dropdownSD, 100)
	
	local dropdownTK = CreateFrame("Frame", "dropdownTK", panel, "UIDropDownMenuTemplate")
	dropdownTK:SetPoint("LEFT", dropdownSK, "LEFT", 0, -50)
	UIDropDownMenu_SetWidth(dropdownTK, 70)
	
	local dropdownTD = CreateFrame("Frame", "dropdownTD", panel, "UIDropDownMenuTemplate")
	dropdownTD:SetPoint("LEFT", dropdownSD, "LEFT", 0, -50)
	UIDropDownMenu_SetWidth(dropdownTD, 100)
	
	local labelKey = dropdownSK:CreateFontString()
	labelKey:SetFontObject(GameFontHighlight)
	labelKey:SetText"Toggle Key"
	labelKey:SetPoint("CENTER", dropdownSK, "CENTER", 0, 40)
	
	local labelDef = dropdownSK:CreateFontString()
	labelDef:SetFontObject(GameFontHighlight)
	labelDef:SetText"Default"
	labelDef:SetPoint("CENTER", dropdownSD, "CENTER", 0, 40)
	
	local dropdownSort = CreateFrame("Frame", "dropdownSort", panel, "UIDropDownMenuTemplate")
	dropdownSort:SetPoint("LEFT", dropdownTD, "LEFT", 0, -50)
	UIDropDownMenu_SetWidth(dropdownSort, 100)

	local labelSort = dropdownSort:CreateFontString()
	labelSort:SetFontObject(GameFontHighlight)
	labelSort:SetText"Sort By"
	labelSort:SetPoint("LEFT", labelToggle, "LEFT", 0, -50)
	
	panel:SetScript("OnShow",bwOnShow)

end

function bwOnShow()
	local keyItems = {"Alt", "Ctrl", "Shift", "None"}
	local sdItems = {"Always Show", "Don't Show"}
	local tdItems = {"Having", "Missing"}
	local sortItems = {"Class", "Group (Short)", "Group (Long)"}

	local function bwOnClick(self, parentDropdownFrame)
		UIDropDownMenu_SetSelectedID(parentDropdownFrame, self:GetID())
		BuffWhoonDB[self.arg2] = self:GetID()
	 end
	
	local function initializeSK(self)
		local info = UIDropDownMenu_CreateInfo()
		for k,v in pairs(keyItems) do
			info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.value = v
			info.arg1 = self
			info.arg2 = "SK"
			info.func = bwOnClick
		UIDropDownMenu_AddButton(info)
		end
	end
	
	local function initializeSD(self)
	   local info = UIDropDownMenu_CreateInfo()
	   for k,v in pairs(sdItems) do
		  info = UIDropDownMenu_CreateInfo()
		  info.text = v
		  info.value = v
		  info.arg1 = self
  		  info.arg2 = "SD"
		  info.func = bwOnClick
		  UIDropDownMenu_AddButton(info)
	   end
	end
	
	local function initializeTK(self)
	   local info = UIDropDownMenu_CreateInfo()
	   for k,v in pairs(keyItems) do
		  info = UIDropDownMenu_CreateInfo()
		  info.text = v
		  info.value = v
		  info.arg1 = self
  		  info.arg2 = "TK"
		  info.func = bwOnClick
		  UIDropDownMenu_AddButton(info)
	   end
	end
	
	local function initializeTD(self)
	   local info = UIDropDownMenu_CreateInfo()
	   for k,v in pairs(tdItems) do
		  info = UIDropDownMenu_CreateInfo()
		  info.text = v
		  info.value = v
		  info.arg1 = self
  		  info.arg2 = "TD"
		  info.func = bwOnClick
		  UIDropDownMenu_AddButton(info)
	   end
	end
	
	local function initializeSort(self)
		local info = UIDropDownMenu_CreateInfo()
		for k,v in pairs(sortItems) do
			info = UIDropDownMenu_CreateInfo()
			info.text = v
			info.value = v
			info.arg1 = self
			info.arg2 = "Sort"
			info.func = bwOnClick
		UIDropDownMenu_AddButton(info)
		end
	end
	

	UIDropDownMenu_Initialize(dropdownSK, initializeSK)
	UIDropDownMenu_SetSelectedID(dropdownSK, BuffWhoonDB["SK"])
	
	UIDropDownMenu_Initialize(dropdownTK, initializeTK)
	UIDropDownMenu_SetSelectedID(dropdownTK, BuffWhoonDB["TK"])
	
	UIDropDownMenu_Initialize(dropdownSD, initializeSD)
	UIDropDownMenu_SetSelectedID(dropdownSD, BuffWhoonDB["SD"])
	
	UIDropDownMenu_Initialize(dropdownTD, initializeTD)
	UIDropDownMenu_SetSelectedID(dropdownTD, BuffWhoonDB["TD"])
	
	UIDropDownMenu_Initialize(dropdownSort, initializeSort)
	UIDropDownMenu_SetSelectedID(dropdownSort, BuffWhoonDB["Sort"])

end

panel:Load()
InterfaceOptions_AddCategory(panel)
