--	configuration
local BAR_TEXTURE = "Interface\\AddOns\\oUF_BossBars\\media\\normTex.tga"
local TEXT_FONT = "Interface\\AddOns\\oUF_BossBars\\media\\uf_font.ttf"

local fontcolor = {1, 1, 1, 1}
local hpBarColor = {178/255, 34/255, 34/255, 1}
local ppBarColor = {165/255, 73/255, 23/255, 1}
local bgBarColor = {44/255, 44/255, 44/255, 1}
local ppBgColor = {29/255, 63/255, 72/255}
local fontSize = 16

local ALTERNATE_POWER_INDEX = ALTERNATE_POWER_INDEX

oUF.TagEvents['BossBars:altpower'] = 'UNIT_POWER'
oUF.Tags['BossBars:altpower'] = function(unit)
	local cur = UnitPower(unit, ALTERNATE_POWER_INDEX)
	local max = UnitPowerMax(unit, ALTERNATE_POWER_INDEX)
	if(max > 0 and not UnitIsDeadOrGhost(unit)) then
		return ("%s%%"):format(math.floor(cur/max*100+.5))
	else
		return ''
	end
end

oUF.TagEvents['BossBars:health'] = 'UNIT_HEALTH'
oUF.Tags['BossBars:health'] = function(unit)
		local m = UnitHealthMax(unit)
		if(m == 0) then
			return 0
		else
			return math.floor(UnitHealth(unit)/m*100+.5)
		end
end

------------------------------------------------------------------------------

-- generate the frames
local function CreateUnitFrame(self, unit)
	local BarWidth = 350
	local BarHeight = 25
	
	-- size the overall unit frame
	self:SetSize(BarWidth, BarHeight)

	------------------------
	-- health bar
	------------------------
	local hpBar = CreateFrame("StatusBar", nil, self)
	hpBar:SetPoint("TOPLEFT", self, "TOPLEFT", 1, -1)
	hpBar:SetSize(BarWidth - 2, BarHeight - 2)
	hpBar:SetStatusBarTexture(BAR_TEXTURE)
	hpBar:GetStatusBarTexture():SetHorizTile(true)
	hpBar:SetStatusBarColor(unpack(hpBarColor))	
	hpBar:SetFrameStrata("MEDIUM")
	
	local hpBG = hpBar:CreateTexture(nil, "BORDER")
	hpBG:SetAllPoints(hpBar)
	hpBG:SetSize(BarWidth - 2, BarHeight - 2)
	hpBG:SetTexture(BAR_TEXTURE)
	hpBG:SetVertexColor(unpack(bgBarColor))	
	
	local hpBorder = hpBar:CreateTexture(nil, "BACKGROUND")
	hpBorder:SetPoint("TOPLEFT", hpBar, "TOPLEFT", -1, 1)
	hpBorder:SetSize(BarWidth, BarHeight)
	hpBorder:SetTexture(0, 0, 0, 1)
	
	-- > see "oUF/elements/health.lua" for health bar coloring options
	hpBar.frequentUpdates = true
	hpBar.colorHealth = false
	self.Health = hpBar
	
	------------------------
	-- power bar
	------------------------
	local ppBar = CreateFrame("StatusBar", nil, self)
	ppBar:SetPoint("BOTTOMLEFT", hpBar, "BOTTOMLEFT", 5, -5)
	ppBar:SetSize(BarWidth - 2, BarHeight - 2)
	ppBar:SetStatusBarTexture(BAR_TEXTURE)	
	ppBar:GetStatusBarTexture():SetHorizTile(true)
	ppBar:SetStatusBarColor(unpack(ppBarColor))
	ppBar:SetFrameStrata("LOW")
	
	local ppBG = ppBar:CreateTexture(nil, "BORDER")
	ppBG:SetAllPoints(ppBar)
	ppBG:SetSize(BarWidth - 2, BarHeight - 2)
	ppBG:SetTexture(BAR_TEXTURE)
	ppBG:SetVertexColor(unpack(ppBgColor))
	
	local ppBorder = ppBar:CreateTexture(nil, "BACKGROUND")
	ppBorder:SetPoint("TOPLEFT", ppBar, "TOPLEFT", -1, 1)
	ppBorder:SetSize(BarWidth, BarHeight)
	ppBorder:SetTexture(0, 0, 0, 1)
	
	-- > see "oUF/elements/power.lua" for power bar coloring options
	ppBar.frequentUpdates = true
	ppBar.colorPower = false
	self.Power = ppBar
	
	------------------------
	-- unit name text
	------------------------

	local Name = hpBar:CreateFontString(nil, "Overlay")
	Name:SetFont(TEXT_FONT, fontSize)
	Name:SetShadowOffset(0,-1)
	Name:SetTextColor(unpack(fontcolor))
	Name:SetJustifyH("LEFT")
	Name:SetPoint("TOPLEFT", hpBar, "TOPLEFT", 5, -4)
	
	-- > see "oUF/elements/tags.lua" for provided tags
	self:Tag(Name, "[name]")
	
	------------------------
	-- unit health text
	------------------------
	local hpText = hpBar:CreateFontString(nil, "Overlay")
	hpText:SetFont(TEXT_FONT, fontSize)
	hpText:SetShadowOffset(.8,-.8)
	hpText:SetTextColor(unpack(fontcolor))
	hpText:SetJustifyH("RIGHT")
	hpText:SetPoint("RIGHT", hpBar, "RIGHT", -4, 0)
	
	-- > "see oUF/elements/tags.lua" for provided tags
	self:Tag(hpText, "[BossBars:health]%")
	
	-- alt power bar
	local AltPowerBar = CreateFrame("StatusBar", nil, self)
	AltPowerBar:SetFrameLevel(self.Health:GetFrameLevel() + 1)
	AltPowerBar:SetSize(98, 13)
	AltPowerBar:SetStatusBarTexture(BAR_TEXTURE)
	AltPowerBar:GetStatusBarTexture():SetHorizTile(false)
	AltPowerBar:SetStatusBarColor(1, 0, 0)
	AltPowerBar:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", 0, 5)
	self.AltPowerBar = AltPowerBar
	
	local bgAltPowerBar = AltPowerBar:CreateTexture(nil, "BORDER")
	bgAltPowerBar:SetAllPoints(AltPowerBar)
	bgAltPowerBar:SetTexture(BAR_TEXTURE)
	bgAltPowerBar:SetVertexColor(unpack(bgBarColor))
	
	local borderAltPowerBar = AltPowerBar:CreateTexture(nil, "BACKGROUND")
	borderAltPowerBar:SetPoint("TOPLEFT", AltPowerBar, "TOPLEFT", -1, 1)
	borderAltPowerBar:SetSize(100.1, 15.1)
	borderAltPowerBar:SetTexture(0, 0, 0, 1)
	
	-- alt power bar text
	local pbText = AltPowerBar:CreateFontString(nil, "Overlay")
	pbText:SetFont(TEXT_FONT, fontSize)
	pbText:SetShadowOffset(.8,-.8)
	pbText:SetTextColor(unpack(fontcolor))
	pbText:SetJustifyH("RIGHT")
	pbText:SetPoint("RIGHT", AltPowerBar, "RIGHT", -4, 0)
	
	self:Tag(pbText, "[BossBars:altpower]")

	--[[
	-- create buff at left of unit if they are boss units
	local buffs = CreateFrame("Frame", nil, self)
	buffs:SetHeight(26)
	buffs:SetWidth(252)
	buffs:SetPoint("RIGHT", self, "LEFT", TukuiDB.Scale(-4), 0)
	buffs.size = 26
	buffs.num = 3
	buffs.spacing = 2
	buffs.initialAnchor = 'RIGHT'
	buffs["growth-x"] = "LEFT"
	buffs.PostCreateIcon = TukuiDB.PostCreateAura
	buffs.PostUpdateIcon = TukuiDB.PostUpdateAura
	self.Buffs = buffs
	]]
end

local function CreateTargetFrame(self, unit)
	local BarWidth = 175
	local BarHeight = 11	
	
	self:SetSize(BarWidth, BarHeight)
	
	local hpBar = CreateFrame("StatusBar", nil, self)
	hpBar:SetStatusBarTexture(BAR_TEXTURE)
	hpBar:GetStatusBarTexture():SetHorizTile(true)	
	hpBar:SetStatusBarColor(unpack(hpBarColor))
	hpBar:SetSize(BarWidth - 2, BarHeight - 2)
	hpBar:SetPoint("TOPLEFT", self, "TOPLEFT", 1, -1)
	
	local hpBG = hpBar:CreateTexture(nil, "BORDER")
	hpBG:SetAllPoints(hpBar)
	hpBG:SetSize(BarWidth - 2, BarHeight - 2.1)
	hpBG:SetTexture(BAR_TEXTURE)
	hpBG:SetVertexColor(unpack(bgBarColor))

	local hpBorder = hpBar:CreateTexture(nil, "BACKGROUND")
	hpBorder:SetPoint("TOPLEFT", hpBar, "TOPLEFT", -1, 1)
	hpBorder:SetSize(BarWidth, BarHeight)
	hpBorder:SetTexture(0, 0, 0, 1)
	
	-- > see "oUF/elements/health.lua" for health bar coloring options
	hpBar.frequentUpdates = true
	hpBar.colorHealth = false
	self.Health = hpBar
	
	local Name = hpBar:CreateFontString(nil, "Overlay")
	Name:SetFont(TEXT_FONT, 13)
	Name:SetShadowOffset(.8,-.8)
	Name:SetTextColor(unpack(fontcolor))
	Name:SetJustifyH("LEFT")
	Name:SetPoint("CENTER", hpBar, "BOTTOM", 0, 2)
	
	-- > see "oUF/elements/tags.lua" for provided tags
	self:Tag(Name, "[name]")	
end

------------------------------------------------------------------------------

oUF:RegisterStyle("oUF_BossBars", CreateUnitFrame)
oUF:RegisterStyle("oUF_BossTargets", CreateTargetFrame)

oUF:SetActiveStyle("oUF_BossBars")
local boss1 = oUF:Spawn("boss1", "oUF_Boss1")
boss1:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 250.1, 085.1)

local boss2 = oUF:Spawn("boss2", "oUF_Boss2")
boss2:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 250.1, 150.1)

local boss3 = oUF:Spawn("boss3", "oUF_Boss3")
boss3:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 250.1, 215.1)

local boss4 = oUF:Spawn("boss4", "oUF_Boss4")
boss4:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 250.1, 285.1)

oUF:SetActiveStyle("oUF_BossTargets")
local boss1T = oUF:Spawn("boss1target", "oUF_Boss1T")
boss1T:SetPoint("BOTTOMLEFT", oUF_Boss1, "TOPLEFT", 0, 5)

local boss2T = oUF:Spawn("boss2target", "oUF_Boss2T")
boss2T:SetPoint("BOTTOMLEFT", oUF_Boss2, "TOPLEFT", 0, 5)

local boss3T = oUF:Spawn("boss3target", "oUF_Boss3T")
boss3T:SetPoint("BOTTOMLEFT", oUF_Boss3, "TOPLEFT", 0, 5)

local boss4T = oUF:Spawn("boss4target", "oUF_Boss4T")
boss4T:SetPoint("BOTTOMLEFT", oUF_Boss4, "TOPLEFT", 0, 5)